import { A } from "@solidjs/router";
import { defaultTo, isNil, pick, pipe, prop } from "ramda";
import { Show, mergeProps } from "solid-js";
import classes from "./ActivatableLink.module.css";

/**
 * @typedef {import("@solidjs/router").AnchorProps} AnchorProps
 * @typedef {import("solid-js").ParentProps} ParentProps
 * @typedef {Partial<AnchorProps> & ParentProps} ActivatableLinkBase
 *
 * @typedef {Object} ActivatableLinkExtends
 * @property {string} href
 * @property {boolean} [activatable]
 * @property {JSX.Element} [leftIcon]
 * @property {JSX.Element} [rightIcon]
 *
 * @typedef {ActivatableLinkBase & ActivatableLinkExtends} ActivatableLinkProps
 */

const defaultActivatable = pipe(defaultTo(true), prop("activatable"));

/**
 * @param {ActivatableLinkProps} props
 * @returns {import("solid-js").Component<ActivatableLinkProps>}
 */
export default function ActivatableLink(props) {
  const mergedProps = mergeProps(
    { activatable: true, leftIcon: null, rightIcon: null },
    props
  );
  return (
    <A
      class={prop("activatable-link", classes)}
      {...(prop("activatable", mergedProps) && {
        activeClass: "activated",
      })}
      {...pick(["replace", "end", "href"], mergedProps)}
    >
      <Show when={!isNil(mergedProps.leftIcon)}>{mergedProps.leftIcon}</Show>
      <Show when={!isNil(mergedProps.children)}>{mergedProps.children}</Show>
      <Show when={!isNil(mergedProps.rightIcon)}>{mergedProps.rightIcon}</Show>
    </A>
  );
}
